@@ -46,9 +46,9 @@ echo " OK"
 
 
 echo -n "Test done authentication (next) ..."
-anastasis-reducer next resources/04-backup.json $TFILE
+anastasis-reducer next resources/04-backup.json "$TFILE"
 
-STATE=`jq -r -e .backup_state < $TFILE`
+STATE=$(jq -r -e .backup_state < "$TFILE")
 if test "$STATE" != "POLICIES_REVIEWING"
 then
     exit_fail "Expected new state to be AUTHENTICATIONS_EDITING, got $STATE"
