@@ -67,8 +67,8 @@ CONF_4="test_anastasis_reducer_free_4.conf"
 
 # Configuration file will be edited, so we create one
 # from the template.
-CONF=`mktemp test_reducerXXXXXX.conf`
-cp test_reducer.conf $CONF
+CONF=$(mktemp test_reducerXXXXXX.conf)
+cp test_reducer.conf "$CONF"
 
 TMP_DIR=`mktemp -d keys-tmp-XXXXXX`
 B1FILE=`mktemp test_reducer_stateB1XXXXXX`
@@ -180,7 +180,7 @@ anastasis-reducer -a \
     "instructions": "What is your name?",
     "challenge": "91GPWWR"
     } }' \
-	add_authentication < $B2FILE > $B1FILE 
+	add_authentication < $B2FILE > $B1FILE
 echo -n "."
 # "64S36" encodes "123"
 anastasis-reducer -a \
