@@ -130,15 +130,15 @@
   op (truth, const struct ANASTASIS_Truth *)  \
   op (policy, const struct ANASTASIS_Policy *)  \
   op (provider_salt, const struct ANASTASIS_CRYPTO_ProviderSaltP)  \
-  op (core_secret, const void *)  \
+  op (core_secret, const void)  \
   op (truth_key, const struct ANASTASIS_CRYPTO_TruthKeyP)  \
   op (account_pub, const struct ANASTASIS_CRYPTO_AccountPublicKeyP)  \
   op (account_priv, const struct ANASTASIS_CRYPTO_AccountPrivateKeyP)  \
   op (payment_secret, const struct ANASTASIS_PaymentSecretP)  \
   op (truth_uuid, const struct ANASTASIS_CRYPTO_TruthUUIDP)  \
   op (eks, const struct ANASTASIS_CRYPTO_EncryptedKeyShareP)  \
-  op (code, const char *) \
-  op (filename, const char *)
+  op (code, const char) \
+  op (filename, const char)
 
 
 /**
@@ -552,7 +552,7 @@ enum ANASTASIS_TESTING_SecretShareOption
  * @param id_data ID data to generate user identifier
  * @param core_secret core secret to backup/recover
  * @param core_secret_size size of @a core_secret
- * @param http_status expected HTTP status.
+ * @param want_status expected status.
  * @param sso secret share options
  * @param ... NULL-terminated list of policy create commands
  * @return the command
@@ -566,7 +566,7 @@ ANASTASIS_TESTING_cmd_secret_share (
   const json_t *id_data,
   const void *core_secret,
   size_t core_secret_size,
-  unsigned int http_status,
+  enum ANASTASIS_ShareStatus want_status,
   enum ANASTASIS_TESTING_SecretShareOption sso,
   ...);
 
