@@ -821,7 +821,7 @@ policy_lookup_cb (void *cls,
     struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_fixed_auto ("uuid",
                                    &cs->ci.uuid),
-      GNUNET_JSON_spec_string ("url",
+      TALER_JSON_spec_web_url ("url",
                                &url),
       GNUNET_JSON_spec_string ("instructions",
                                &instructions),
@@ -1150,7 +1150,7 @@ parse_cs_array (struct ANASTASIS_Recovery *r,
     struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_fixed_auto ("uuid",
                                    &c->ci.uuid),
-      GNUNET_JSON_spec_string ("url",
+      TALER_JSON_spec_web_url ("url",
                                &url),
       GNUNET_JSON_spec_string ("instructions",
                                &instructions),
@@ -1355,7 +1355,7 @@ ANASTASIS_recovery_deserialize (struct GNUNET_CURL_Context *ctx,
     struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_fixed_auto ("id",
                                    &r->id),
-      GNUNET_JSON_spec_string ("provider_url",
+      TALER_JSON_spec_web_url ("provider_url",
                                &provider_url),
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_string ("secret_name",
