@@ -2308,12 +2308,12 @@ done_secret_selecting (json_t *state,
                        void *cb_cls)
 {
   uint32_t mask;
-  json_t *pa;
+  const json_t *pa;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_uint32 ("attribute_mask",
                              &mask),
-    GNUNET_JSON_spec_json ("providers",
-                           &pa),
+    GNUNET_JSON_spec_array_const ("providers",
+                                  &pa),
     GNUNET_JSON_spec_end ()
   };
   struct ANASTASIS_CRYPTO_ProviderSaltP provider_salt;
@@ -2448,7 +2448,6 @@ done_secret_selecting (json_t *state,
         pd->ra.cleanup_cls = pd;
         return &pd->ra;
       }
-
     }
   }
 
