@@ -139,13 +139,9 @@ policy_store_cb (void *cls,
   pss->pso = NULL;
   if (ud->http_status != pss->http_status)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u to command %s in %s:%u\n",
-                ud->http_status,
-                pss->is->commands[pss->is->ip].label,
-                __FILE__,
-                __LINE__);
-    TALER_TESTING_interpreter_fail (pss->is);
+    TALER_TESTING_unexpected_status (pss->is,
+                                     ud->http_status,
+                                     pss->http_status);
     return;
   }
   switch (ud->us)
@@ -276,7 +272,7 @@ policy_store_run (void *cls,
                       pss->recovery_data_size,
                       &pss->curr_hash);
   pss->pso = ANASTASIS_policy_store (
-    is->ctx,
+    TALER_TESTING_interpreter_get_context (is),
     pss->anastasis_url,
     &pss->anastasis_priv,
     pss->recovery_data,
@@ -340,8 +336,7 @@ policy_store_traits (void *cls,
   struct PolicyStoreState *pss = cls;
   struct TALER_TESTING_Trait traits[] = {
     TALER_TESTING_make_trait_claim_token (&pss->claim_token),
-    TALER_TESTING_make_trait_order_id (
-      (const char **) &pss->order_id),
+    TALER_TESTING_make_trait_order_id (pss->order_id),
     ANASTASIS_TESTING_make_trait_hash (&pss->curr_hash),
     ANASTASIS_TESTING_make_trait_account_pub (&pss->anastasis_pub),
     ANASTASIS_TESTING_make_trait_account_priv (&pss->anastasis_priv),
