@@ -74,7 +74,7 @@ struct RecoverSecretState
   /**
    * Coresecret to check if decryption worked
    */
-  const void **core_secret;
+  const void *core_secret;
 
   /**
    * Task scheduled to wait for recovery to complete.
@@ -108,13 +108,19 @@ policy_lookup_cb (void *cls,
 {
   struct RecoverSecretState *rss = cls;
 
-  rss->ri = (struct ANASTASIS_RecoveryInformation *) ri;
   if (NULL == ri)
   {
     GNUNET_break (0);
     TALER_TESTING_interpreter_fail (rss->is);
     return;
   }
+  if (0 == ri->cs_len)
+  {
+    GNUNET_break (0);
+    TALER_TESTING_interpreter_fail (rss->is);
+    return;
+  }
+  rss->ri = (struct ANASTASIS_RecoveryInformation *) ri;
   TALER_TESTING_interpreter_next (rss->is);
 }
 
@@ -146,7 +152,7 @@ core_secret_cb (void *cls,
     return;
   }
   if (0 != memcmp (secret,
-                   *rss->core_secret,
+                   rss->core_secret,
                    secret_size))
   {
     GNUNET_break (0);
@@ -188,9 +194,9 @@ recover_secret_run (void *cls,
 
   if (NULL != rss->download_reference)
   {
-    ref = TALER_TESTING_interpreter_lookup_command
-            (is,
-            rss->download_reference);
+    ref = TALER_TESTING_interpreter_lookup_command (
+      is,
+      rss->download_reference);
     if (NULL == ref)
     {
       GNUNET_break (0);
@@ -220,22 +226,23 @@ recover_secret_run (void *cls,
     if (GNUNET_OK !=
         ANASTASIS_TESTING_get_trait_core_secret (
           ref,
-          (const void ***) &rss->core_secret))
+          &rss->core_secret))
     {
       GNUNET_break (0);
       TALER_TESTING_interpreter_fail (rss->is);
       return;
     }
   }
-  rss->recovery = ANASTASIS_recovery_begin (is->ctx,
-                                            rss->id_data,
-                                            rss->version,
-                                            rss->anastasis_url,
-                                            provider_salt,
-                                            &policy_lookup_cb,
-                                            rss,
-                                            &core_secret_cb,
-                                            rss);
+  rss->recovery = ANASTASIS_recovery_begin (
+    TALER_TESTING_interpreter_get_context (is),
+    rss->id_data,
+    rss->version,
+    rss->anastasis_url,
+    provider_salt,
+    &policy_lookup_cb,
+    rss,
+    &core_secret_cb,
+    rss);
   if (NULL == rss->recovery)
   {
     GNUNET_break (0);
@@ -301,7 +308,7 @@ recover_secret_cleanup (void *cls,
  * @param index index number of the object to extract.
  * @return #GNUNET_OK on success
  */
-static int
+static enum GNUNET_GenericReturnValue
 recover_secret_traits (void *cls,
                        const void **ret,
                        const char *trait,
